/*
 *  Copyright (C) 2007 Baryshev Dmitry  <ksquirrel.iv@gmail.com>
 *
 *  for KConnections project
 */

/*****************************************************************

Copyright (c) 2004 Zack Rusin <zrusin@kde.org>
                   Sami Kyostil <skyostil@kempele.fi>
                   Aaron J. Seigo <aseigo@kde.org>

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

******************************************************************/

#include <qapplication.h>
#include <qpainter.h>
#include <qsimplerichtext.h>
#include <qtimer.h>

#include <kdialog.h>
#include <kwin.h>

#include "kickertip.h"

KickerTip::KickerTip(QWidget *parent)
    : QWidget(parent, "animtt"), m_richText(0)
{
    KWin::setType(winId(), NET::Override);

    setFocusPolicy(NoFocus);
    setBackgroundMode(NoBackground);
    resize(0, 0);
    hide();
}

void KickerTip::display(const QString& title)
{
    delete m_richText;
    m_richText = new QSimpleRichText(title, font());

    displayInternal();
}

void KickerTip::paintEvent(QPaintEvent *e)
{
    QPainter p(this);
    p.drawPixmap(e->rect().topLeft(), m_pixmap, e->rect());
}

void KickerTip::mousePressEvent(QMouseEvent * /*e*/)
{
    hide();
}

void KickerTip::plainMask()
{
    QPainter maskPainter(&m_mask);

    m_mask.fill(Qt::black);

    maskPainter.setBrush(Qt::white);
    maskPainter.setPen(Qt::white);
    maskPainter.drawRoundRect(m_mask.rect(), 1600 / m_mask.rect().width(),
                              1600 / m_mask.rect().height());
    setMask(m_mask);
}

void KickerTip::displayInternal()
{
    if (!m_richText)
        return;

    KWin::setState(winId(), NET::Sticky | NET::SkipTaskbar | NET::SkipPager | NET::KeepAbove);
    KWin::setOnAllDesktops(winId(), true);

    // determine text rectangle
    QRect textRect(0, 0, m_richText->widthUsed(), m_richText->height());
    textRect.setHeight(m_richText->height()-16);
    textRect.moveBy(-textRect.left(), -textRect.top());
    textRect.addCoords(0, 0, 2, 2);

    int textX = 2;
    int width = textX + textRect.width() + KDialog::marginHint();
    int height = textRect.height();
    int textY = (height - textRect.height()) / 2;

    // resize pixmap, mask and widget
    m_mask.resize(width, height);
    m_pixmap.resize(width, height);
    resize(width, height);

    plainMask();

    // draw background
    QPainter bufferPainter(&m_pixmap);
    bufferPainter.setPen(Qt::white);
    bufferPainter.setBrush(Qt::white);
    bufferPainter.drawRoundRect(0, 0, width, height,
                                1600 / width, 1600 / height);

    // draw text
    m_richText->draw(&bufferPainter, 5 + textX, textY, rect(), colorGroup());
}

#include "kickertip.moc"

