/***************************************************************************
                          kwlabel.cpp  -  description
                             -------------------
    begin                : ??? Dec 5 2007
    copyright            : (C) 2007 by Krasu
    email                : ksquirrel.iv@gmail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qtimer.h>
#include <qeventloop.h>

#include <kapplication.h>

#include "kwlabel.h"
#include "kwentry.h"
#include "kwconfig.h"
#include "kickertip.h"

KWLabel::KWLabel(QWidget *parent) : KickerTip(parent)
{
    m_x = m_y = 0;

    timer = new QTimer(this);
    connect(timer, SIGNAL(timeout()), this, SLOT(slotTimer()));

    rereadSettings();
}

KWLabel::~KWLabel()
{}

void KWLabel::addText(const Entry &e, MessageType tp)
{
    LabelEntry le;

    // we need a copy of entry, since it can be deleted
    // while we updating this label.
    //
    // pointer is too dangerous
    le.e = e;
    le.message = tp;

    labels.append(le);

    if(!updateText())
        return;

    move(m_x, m_y-height());
    show();

    KApplication::eventLoop()->processEvents(QEventLoop::ExcludeUserInput|QEventLoop::ExcludeSocketNotifiers);

    timer->start(delay, true);
}

void KWLabel::rereadSettings()
{
    bgcolor_con = "";
    bgcolor_discon = "";

    KWConfig::instance()->setGroup("General");

    if(KWConfig::instance()->readBoolEntry("messages_connected_color", true))
        bgcolor_con = QString::fromLatin1("bgcolor='%1'").arg(KWConfig::instance()->readEntry("messages_connected_color_color", "#59FF83"));

    if(KWConfig::instance()->readBoolEntry("messages_disconnected_color", true))
        bgcolor_discon = QString::fromLatin1("bgcolor='%1'").arg(KWConfig::instance()->readEntry("messages_disconnected_color_color", "#FF6565"));

    delay = KWConfig::instance()->readNumEntry("message_delay", 2000);

    incoming = KWConfig::instance()->readBoolEntry("messages_incoming", true);
    outgoing = KWConfig::instance()->readBoolEntry("messages_outgoing", true);

    if(delay < 0 || delay > 10000)
        delay = 2000;
}

bool KWLabel::updateText()
{
    QString result = "<table margin=0 cellspacing=0 cellpadding=0>";
    Labels::iterator itEnd = labels.end();
    int cnt = 0;

    for(Labels::iterator it = labels.begin();it != itEnd;++it)
    {
        if(((*it).e.incoming && !incoming) || (!(*it).e.incoming && !outgoing))
            continue;

        cnt++;

        result += QString::fromLatin1("<tr %1>").arg((*it).message == Added ? bgcolor_con : bgcolor_discon);
        result += (*it).e.tableEntry();
        result += "</tr>\n";
    }

    result += "</table>\n";

    display(result);

    return cnt;
}

void KWLabel::slotTimer()
{
    hide();
    labels.clear();
}

#include "kwlabel.moc"
