/****************************************************************************
** ui.h extension file, included from the uic-generated form implementation.
**
** If you want to add, delete, or rename functions or slots, use
** Qt Designer to update this file, preserving your code.
**
** You should not define a constructor or destructor in this file.
** Instead, write your code in functions called init() and destroy().
** These will automatically be called by the form's constructor and
** destructor.
*****************************************************************************/

class KWTableItemIP : public QTableItem
{
public:
    KWTableItemIP(QTable *table, EditType et) : QTableItem(table, et)
    {}
    ~KWTableItemIP()
    {}

    QWidget* createEditor() const
    {
	QLineEdit *e = (QLineEdit *)QTableItem::createEditor();
	e->setInputMask("000.000.000.000; ");
	return e;
    }
};

class KWTableItemCheck : public QTableItem
{
public:
    KWTableItemCheck(QTable *table, EditType et) : QTableItem(table, et)
    {
	setChecked(true);
    }
    ~KWTableItemCheck()
    {}

    void setChecked(bool b)
    {
	m_checked = b;
	QPixmap p;
	QPixmapCache::find(b ? "ignore_enabled" : "ignore_disabled", p);

	setPixmap(p);

	if(table())
	    table()->updateCell(row(), col());
    }

    bool checked() const
    {
	return m_checked;
    }

private:
    bool m_checked;
};

/******************************************/

void KWOptions::init()
{
    connect(listMain, SIGNAL(highlighted(int)), widgetStack, SLOT(raiseWidget(int)));

    pcheck = KGlobal::iconLoader()->loadIcon("ok", KIcon::Desktop, KIcon::SizeSmall);

    QPixmapCache::insert("ignore_enabled", pcheck);
    QPixmapCache::insert("ignore_disabled", puncheck);

    new KWIconListItem(listMain, KGlobal::iconLoader()->loadIcon("network", KIcon::Desktop, KIcon::SizeMedium), i18n("General"));    
    new KWIconListItem(listMain, KGlobal::iconLoader()->loadIcon("list", KIcon::Desktop, KIcon::SizeMedium), i18n("Ignore list"));    

    KWConfig *konf = KWConfig::instance();

    konf->setGroup("General");
    checkIncoming->setChecked(konf->readBoolEntry("messages_incoming", true));
    checkOutgoing->setChecked(konf->readBoolEntry("messages_outgoing", true));
    checkMessages->setChecked(konf->readBoolEntry("messages", true));
    checkConnected->setChecked(konf->readBoolEntry("messages_connected", true));
    checkDisconnected->setChecked(konf->readBoolEntry("messages_disconnected", true));
    checkConnectedColor->setChecked(konf->readBoolEntry("messages_connected_color", true));
    checkDisconnectedColor->setChecked(konf->readBoolEntry("messages_disconnected_color", true));
    int id = konf->readNumEntry("ignore_type", 3);
    buttonGroupList->setButton(id);

    QColor color;
    color.setNamedColor(konf->readEntry("messages_connected_color_color", "#59FF83"));
    colorConnected->setColor(color);
    color.setNamedColor(konf->readEntry("messages_disconnected_color_color", "#FF6565"));
    colorDisconnected->setColor(color);

    rescanDelay->setRange(0, 30000, 500, true);
    rescanDelay->setValue(konf->readNumEntry("rescan_delay", 1000));

    messageDelay->setRange(0, 10000, 100, true);
    messageDelay->setValue(konf->readNumEntry("message_delay", 2000));
    lineFormat->setText(konf->readEntry("message_format", "<td><b>%p</b></td> <td><b>%s</b></td> <td><b>%us</b></td> <td>%l:%lp</td> <td>%i</td> <td>%r:%rp</td>"));

    listMain->updateAndInstall(this);

    konf->setGroup("Main");
    listMain->setCurrentItem(konf->readNumEntry("options_last", 0));

    initLists();

    konf->setGroup("Ignore");
    table = tableIgnore;
    table->adjustColumn(0);
    readIgnoreList();

    konf->setGroup("Accept");
    table = tableAccept;
    table->adjustColumn(0);
    readIgnoreList();

    slotIgnoreTypeChanged(id);
}

void KWOptions::initLists()
{
    passwd *pwd;
    servent *serv;

    setpwent();
    while((pwd = getpwent()))
    {
        if(listUser.find(pwd->pw_name) == listUser.end())
    	    listUser << pwd->pw_name;
    }

    setservent(0);
    while((serv = getservent()))
    {
        if(listServ.find(serv->s_name) == listServ.end())
    	    listServ << serv->s_name;

        if(listProt.find(serv->s_proto) == listProt.end())
	    listProt << serv->s_proto;
    }

    listUser.sort();
    listServ.sort();
    listProt.sort();

    listUser.prepend("$me");
    listUser.prepend("-"); // '-' => don't use this type

    listServ.prepend("-");
    listProt.prepend("-");
}

void KWOptions::readIgnoreList()
{
    KWConfig *konf = KWConfig::instance();
    QStringList list;
    QStringList::iterator it, itEnd;
    int i = 0;

    while(true)
    {
	list = konf->readListEntry(QString::number(i));

	if(list.count() < 3)
	    break;

	slotIgnorePlus();
	KWTableItemCheck *itemEnab = (KWTableItemCheck *)table->item(i, 0);
	QComboTableItem *itemProt = (QComboTableItem *)table->item(i, 1);
	QComboTableItem *itemServ = (QComboTableItem *)table->item(i, 2);
	QComboTableItem *itemUser = (QComboTableItem *)table->item(i, 3);
	QTableItem *itemP = table->item(i, 4);
	QTableItem *itemLA = table->item(i, 5);
	QTableItem *itemLAP = table->item(i, 6);
	QTableItem *itemRA = table->item(i, 7);
	QTableItem *itemRAP = table->item(i, 8);

	it = list.begin();
	itEnd = list.end();

	if((*it).at(0).isNumber())
	{
	    itemEnab->setChecked((*it == "1"));
	}
	else
	    itemEnab->setChecked(false);

	++it;

	itemProt->setCurrentItem(*it);
	++it;

	itemServ->setCurrentItem(*it);
	++it;

	itemUser->setCurrentItem(*it);
	++it;

	if(it != itEnd)
	{
	    itemP->setText(*it);
	    ++it;
	}

	if(it != itEnd)
	{
	    itemLA->setText(*it);
	    ++it;
	}

	if(it != itEnd)
	{
	    itemLAP->setText(*it);
	    ++it;
	}

	if(it != itEnd)
	{
	    itemRA->setText(*it);
	    ++it;
	}

	if(it != itEnd)
	    itemRAP->setText(*it);

        // enable/disable rows
        if(!itemEnab->checked())
	    for(int c = 1;c < 9;c++)
	        table->item(i, c)->setEnabled(false);

	i++;
    }

    adjustColumns();
}

void KWOptions::writeIgnoreList()
{
    KWConfig *konf = KWConfig::instance();
    int rows = table->numRows();
    QStringList list;
    int i = 0;

    while(i < rows)
    {
	KWTableItemCheck *itemEnab = (KWTableItemCheck *)table->item(i, 0);
	QComboTableItem *itemProt = (QComboTableItem *)table->item(i, 1);
	QComboTableItem *itemServ = (QComboTableItem *)table->item(i, 2);
	QComboTableItem *itemUser = (QComboTableItem *)table->item(i, 3);
	QTableItem *itemP = table->item(i, 4);
	QTableItem *itemLA = table->item(i, 5);
	QTableItem *itemLAP = table->item(i, 6);
	QTableItem *itemRA = table->item(i, 7);
	QTableItem *itemRAP = table->item(i, 8);

	list << (itemEnab->checked() ? "1" : "0");
	list << itemProt->currentText();
	list << itemServ->currentText();
	list << itemUser->currentText();
	list << itemP->text();
	list << itemLA->text();
	list << itemLAP->text();
	list << itemRA->text();
	list << itemRAP->text();

	konf->writeEntry(QString::number(i), list);
	list.clear();

	i++;
    }
}

int KWOptions::start()
{
    int result = QDialog::exec();

    if(result == QDialog::Accepted)
    {
	KWConfig *konf = KWConfig::instance();

	konf->setGroup("Main");
	konf->writeEntry("options_last", listMain->currentItem());

	konf->setGroup("General");
	konf->writeEntry("messages_incoming", checkIncoming->isChecked());
	konf->writeEntry("messages_outgoing", checkOutgoing->isChecked());
	konf->writeEntry("messages", checkMessages->isChecked());
	konf->writeEntry("messages_connected", checkConnected->isChecked());
	konf->writeEntry("messages_connected_color", checkConnectedColor->isChecked());
	konf->writeEntry("messages_connected_color_color", colorConnected->color().name());
	konf->writeEntry("messages_disconnected", checkDisconnected->isChecked());
	konf->writeEntry("messages_disconnected_color", checkDisconnectedColor->isChecked());
	konf->writeEntry("messages_disconnected_color_color", colorDisconnected->color().name());
	konf->writeEntry("message_delay", messageDelay->value());
	konf->writeEntry("rescan_delay", rescanDelay->value());
	konf->writeEntry("message_format", lineFormat->text());
	konf->writeEntry("ignore_type", buttonGroupList->selectedId());

	konf->deleteGroup("Ignore");
	konf->setGroup("Ignore");
	table = tableIgnore;
	writeIgnoreList();

	konf->deleteGroup("Accept");
	konf->setGroup("Accept");
	table = tableAccept;
	writeIgnoreList();
    }

    return result;
}

void KWOptions::slotIgnorePlus()
{    
    table->insertRows(table->numRows());        
    int id = table->numRows() - 1;

    insertPrivate(id);
}

void KWOptions::insertPrivate(int id)
{
    table->setItem(id, 0, new KWTableItemCheck(table, QTableItem::Never));
    table->setItem(id, 1, new QComboTableItem(table, QStringList()));
    table->setItem(id, 2, new QComboTableItem(table, QStringList()));
    table->setItem(id, 3, new QComboTableItem(table, QStringList()));
    table->setItem(id, 4, new QTableItem(table, QTableItem::OnTyping));
    table->setItem(id, 5, new KWTableItemIP(table, QTableItem::OnTyping));
    table->setItem(id, 6, new QTableItem(table, QTableItem::OnTyping));
    table->setItem(id, 7, new KWTableItemIP(table, QTableItem::OnTyping));
    table->setItem(id, 8, new QTableItem(table, QTableItem::OnTyping));

    insertDefaultValues(id);
    adjustColumns();

    table->selectRow(id);
}

void KWOptions::slotIgnoreMinus()
{
    int id = table->currentRow();
    table->removeRow(id);
    table->selectRow(id);
}

void KWOptions::adjustColumns()
{
/*
    table->adjustColumn(4);
    table->adjustColumn(5);
    table->adjustColumn(6);
    table->adjustColumn(7);
    table->adjustColumn(8);
*/
}

void KWOptions::insertDefaultValues(int row)
{
    QComboTableItem *itemProt = (QComboTableItem *)table->item(row, 1);
    QComboTableItem *itemServ = (QComboTableItem *)table->item(row, 2);
    QComboTableItem *itemUser = (QComboTableItem *)table->item(row, 3);

    itemUser->setStringList(listUser);
    itemServ->setStringList(listServ);
    itemProt->setStringList(listProt);
}

void KWOptions::slotCopy()
{
    int id = table->currentRow();

    if(id < 0)
	return;

    int id2 = id+1;
    table->insertRows(id2);
    insertPrivate(id2);

    KWTableItemCheck *itemEnab = (KWTableItemCheck *)table->item(id, 0);
    QComboTableItem *itemProt = (QComboTableItem *)table->item(id, 1);
    QComboTableItem *itemServ = (QComboTableItem *)table->item(id, 2);
    QComboTableItem *itemUser = (QComboTableItem *)table->item(id, 3);
    QTableItem *itemP = table->item(id, 4);
    QTableItem *itemLA = table->item(id, 5);
    QTableItem *itemLAP = table->item(id, 6);
    QTableItem *itemRA = table->item(id, 7);
    QTableItem *itemRAP = table->item(id, 8);

    KWTableItemCheck *itemEnab2 = (KWTableItemCheck *)table->item(id2, 0);
    QComboTableItem *itemProt2 = (QComboTableItem *)table->item(id2, 1);
    QComboTableItem *itemServ2 = (QComboTableItem *)table->item(id2, 2);
    QComboTableItem *itemUser2 = (QComboTableItem *)table->item(id2, 3);
    QTableItem *itemP2 = table->item(id2, 4);
    QTableItem *itemLA2 = table->item(id2, 5);
    QTableItem *itemLAP2 = table->item(id2, 6);
    QTableItem *itemRA2 = table->item(id2, 7);
    QTableItem *itemRAP2 = table->item(id2, 8);

    itemEnab2->setChecked(itemEnab->checked());
    itemProt2->setCurrentItem(itemProt->text());
    itemServ2->setCurrentItem(itemServ->text());
    itemUser2->setCurrentItem(itemUser->text());
    itemP2->setText(itemP->text());
    itemLA2->setText(itemLA->text());
    itemLAP2->setText(itemLAP->text());
    itemRA2->setText(itemRA->text());
    itemRAP2->setText(itemRAP->text());
}

void KWOptions::slotClicked(int row, int col)
{
    if(!col)
    {
	KWTableItemCheck *item = (KWTableItemCheck *)table->item(row, col);

	if(!item) return;

	bool enab =  !item->checked();

	item->setChecked(enab);

	for(int i = 1;i < 9;i++)
	    table->item(row, i)->setEnabled(enab);
    }
}

void KWOptions::slotIgnoreTypeChanged(int id)
{
    widgetStackIgnore->setEnabled(id > 1);

    if(id == 2) // black list
    {
	table = tableIgnore;
	widgetStackIgnore->raiseWidget(0);
    }
    else if(id == 3) // white list
    {
	table = tableAccept;
	widgetStackIgnore->raiseWidget(1);
    }
}
